/********************************************************************************
 *   Prometheus exporter for monitoring network connectivity using icmp pings   *
 *                                                                              *
 *   Copyright (C) 2019-2020 Jan Christian Grünhage                             *
 *   Copyright (C) 2020 Famedly GmbH                                            *
 *                                                                              *
 *   This program is free software: you can redistribute it and/or modify       *
 *   it under the terms of the GNU Affero General Public License as             *
 *   published by the Free Software Foundation, either version 3 of the         *
 *   License, or (at your option) any later version.                            *
 *                                                                              *
 *   This program is distributed in the hope that it will be useful,            *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               *
 *   GNU Affero General Public License for more details.                        *
 *                                                                              *
 *   You should have received a copy of the GNU Affero General Public License   *
 *   along with this program.  If not, see <https://www.gnu.org/licenses/>.     *
 ********************************************************************************/
use anyhow::{Context, Result};

mod config;
mod metrics;
mod ping;

use crate::config::{read_config, setup_clap, setup_fern};
use crate::metrics::start_serving_metrics;
use crate::ping::start_pinging_hosts;

#[tokio::main]
async fn main() -> Result<()> {
    let clap = setup_clap();
    setup_fern(clap.occurrences_of("v"));
    let config =
        read_config(clap.value_of("config").unwrap()).context("Couldn't read config file!")?;

    let ping_fut = start_pinging_hosts(config.clone());
    let serve_fut = start_serving_metrics(config.clone());

    futures::pin_mut!(ping_fut);
    futures::pin_mut!(serve_fut);

    futures::future::select(ping_fut, serve_fut).await;
    Ok(())
}
